import os
from datetime import datetime
from os import path
from socket import gaierror
from sys import stdout

from requests.exceptions import ConnectionError
from termcolor import cprint

from .reference import (ASCII_ART, COLOR, END, LOOKUP_ICON, LOOKUP_WIND, NAME,
                        WARNING_COLOR, ZERO)
from .utils import (C2F, clear_scr, from_future, getch, handle_connectionerror,
                    time_from_now, wind_degrees)
from .weather import Forecast, Weather


class TerminalInterface:
    def __init__(self):
        pass

    def info(self):
        cprint(ASCII_ART, color=COLOR)

    def weather(self, loc=None):
        w = Weather(loc=loc)
        try:
            wttr = w.here.get.weather
            self.display_weather_info(wttr)
        except ConnectionError:
            handle_connectionerror()

    def display_weather_info(self, wttr):
        print("It's {}".format(str(datetime.now()).split('.')[0]))
        try:
            icon = LOOKUP_ICON[wttr["weather"][0]["icon"][:2]]
            temp_C = wttr["main"]["temp"]-ZERO
            cprint("{temp:.1f}".format(temp=temp_C),
                   end='', color=COLOR, flush=True)
            print("°C/", end='', flush=True)
            cprint("{tempF:.1f}".format(tempF=C2F(temp_C)),
                   end='', color=COLOR, flush=True)
            print("°F  {icon}".format(
                icon=icon), end=END)
        except KeyError:
            pass
        print(wttr["weather"][0]["main"], end=END)
        print(wttr["weather"][0]["description"], end=END)
        print("in {country} {loc}".format(
            country=wttr["sys"]["country"],
            loc=(wttr["coord"]["lon"], wttr["coord"]["lat"])), end='')
        print()
        drc = wind_degrees(wttr["wind"]["deg"])
        print("Wind: {deg}  {speed}(m/s)".format(
            deg=LOOKUP_WIND[drc], speed=wttr["wind"]["speed"]), end='')
        print()

    def forecast(self, loc=None):
        f = Forecast(loc=loc)
        try:
            fore = f.here.get.forecast
            self.display_forecast_info(fore)
        except ConnectionError:
            handle_connectionerror()

    def display_forecast_info(self, full_forecast):
        index = 0
        try:
            while not from_future(full_forecast["list"][index]["dt_txt"]):
                index += 1
        except IndexError:
            cprint("No up-to-date data available :(",
                   color=WARNING_COLOR, flush=True)
            return
        start_index = index
        show_entry = True
        while True:
            if show_entry:
                self.print_entry(fore=full_forecast["list"][index],
                                 city=full_forecast["city"])

            cprint("Input", end='', color=COLOR, flush=True)
            print(": [", end='', flush=True)
            cprint("f", end='', color=COLOR, flush=True)
            print("]orward 3 hours, [", end='', flush=True)
            cprint("b", end='', color=COLOR, flush=True)
            print("]ackward 3 hours, [", end='', flush=True)
            cprint("q", end='', color=COLOR, flush=True)
            print("]uit: ", end='', flush=True)
            typed = getch().lower()

            if typed == 'b':
                if index == start_index:
                    cprint("No more entries!\a", end='', color=WARNING_COLOR)
                    print()
                    show_entry = False
                    continue
                index -= 1
                show_entry = True
            elif typed == 'f':
                if index == full_forecast["cnt"] - 1:
                    cprint("No more entries!\a", end='', color=WARNING_COLOR)
                    print()
                    show_entry = False
                    continue
                index += 1
                show_entry = True
            elif typed == 'q' or typed == "quit":
                print(flush=True)
                break
            else:
                cprint('Invalid input\a', end='', color=WARNING_COLOR)
                show_entry = False
                print()
                continue

            clear_scr()

    def print_entry(self, fore, city):
        print("Forecast: {time}{END}{from_now}".format(
            END=END,
            time=fore["dt_txt"],
            from_now=time_from_now(fore["dt_txt"])), end='')
        print()
        try:
            i = fore["weather"][0]["icon"][:2]
            icon = LOOKUP_ICON[i]
            temp_C = fore["main"]["temp"]-ZERO
            cprint("{temp:.1f}".format(temp=temp_C),
                   end='', color=COLOR, flush=True)
            print("°C/", end='', flush=True)
            cprint("{tempF:.1f}".format(tempF=C2F(temp_C)),
                   end='', color=COLOR, flush=True)
            print("°F  {icon}".format(
                icon=icon, temp=temp_C, tempF=C2F(temp_C)), end=END)
        except KeyError:
            pass
        print("{main}".format(main=fore["weather"][0]["main"]), end=END)
        print("{description}".format(
            description=fore["weather"][0]["description"]), end=END)
        print("in {country} ({lon:.1f},{lat:.1f})".format(country=city["country"],
                                                          lon=city["coord"]["lon"], lat=city["coord"]["lat"]), end='')
        print()

        drc = wind_degrees(fore["wind"]["deg"])
        print("Wind: {wind}  {speed}(m/s)".format(wind=LOOKUP_WIND[drc],
                                                  speed=fore["wind"]["speed"]), end='')
        print()



import os
import sys
import termios
import tty
from datetime import datetime, timedelta
from os import path

import requests
from requests.exceptions import ConnectionError


def print_help():
    print(
        '''
usage: kairos [h] [n] [f]

optional arguments:
  h, -h, help, --help      
        show this help message and exit

  n, -n, now, --now       
        Displays the current weather

  f, -f, fore, --fore, forecast, --forecast 
        Displays the weather forecast
''')


def handle_connectionerror():
    print("ConnectionError :: please first connect to the Internet!")
    exit()


# adapted from https://github.com/recantha/EduKit3-RC-Keyboard/blob/master/rc_keyboard.py
def getch():
    fd = sys.stdin.fileno()
    old_settings = termios.tcgetattr(fd)
    try:
        tty.setraw(sys.stdin.fileno())
        ch = sys.stdin.read(1)
    finally:
        termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
    return ch


def clear_scr():
    os.system('clear')


def println():
    print(flush=True)


def wind_degrees(deg):
    return int((8*(deg+360/16)/360) % 8)


def time_from_now(time):
    from_now = datetime.strptime(time, "%Y-%m-%d %H:%M:%S") - datetime.now()
    days = from_now.days
    hours = from_now.seconds // (60*60)
    minutes = (from_now.seconds % (60*60)) // 60
    seconds = from_now.seconds % 60
    dy = "day" if days <= 1 else "days"
    hr = "hour" if hours <= 1 else "hours"
    mn = "minute" if minutes <= 1 else "minutes"
    sc = "second" if seconds <= 1 else "seconds"
    return "{D} {dy}, {H} {hr}, {M} {mn}, and {S} {sc} from now"\
        .format(D=from_now.days, H=hours, M=minutes, S=seconds, dy=dy, hr=hr, mn=mn, sc=sc)


def from_future(time):
    from_now = datetime.strptime(time, "%Y-%m-%d %H:%M:%S") - datetime.now()
    days = from_now.days
    hours = from_now.seconds // (60*60)
    minutes = (from_now.seconds % (60*60)) // 60
    seconds = from_now.seconds % 60
    return all((days >= 0, hours >= 0, minutes >= 0, seconds >= 0))


def C2F(c):
    return c*(9/5)+32



import json

import requests

from .reference import API_KEY, FORECAST, IP_INFO, WEATHER


class Location:
    def __init__(self):
        pass

    @property
    def get(self):
        # ignores exceptions by design
        r = requests.get(IP_INFO)
        r = json.loads(r.text)

        self.region = r["region"]
        self.city = r["city"]
        self.country = r["country"]
        self.location = tuple(float(f) for f in r["loc"].split(','))

        return self


class Weather:
    def __init__(self, loc=None):
        self.location = Location()
        self.loc = loc

    @property
    def get(self):
        lat, lon = self.loc if self.loc else \
            tuple(int(i) for i in self.location.get.location)

        r = requests.get(url=WEATHER.format(lat=lat, lon=lon, api=API_KEY))
        self.weather = json.loads(r.text)
        return self

    @property
    def here(self):
        self.location.get
        return self


class Forecast:
    def __init__(self, loc=None):
        self.location = Location()
        self.loc = loc

    @property
    def get(self):
        lat, lon = self.loc if self.loc else \
            tuple(int(i) for i in self.location.get.location)

        r = requests.get(url=FORECAST.format(lat=lat, lon=lon, api=API_KEY))
        self.forecast = json.loads(r.text)
        return self

    @property
    def here(self):
        self.location.get
        return self



import setuptools

with open("README.md", "r") as rm:
    long_description = rm.read()

setuptools.setup(
    name='wkairos',
    version='0.0',
    scripts=['kairos'],
    author="MutatedFlood",
    author_email="b06901038@g.ntu.edu.tw",
    description="A command line weather app.",
    install_requires=["requests", "termios"],
    license="MIT",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="weather",
    url="https://github.com/MutatedFlood/KaiRoS",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
